"""Build and parse standard GitHub URLs.
"""

__all__ = ('RepoSlug', 'parse_repo_slug_from_url', 'make_raw_content_url',
           'normalize_repo_root_url')

import collections
import re


RepoSlug = collections.namedtuple('RepoSlug', 'full owner repo')
"""GitHub repository slug (`collections.namedtuple`).

Attributes
----------
full : `str`
    Full repository slug. Example: ``'lsst-sqre/lsst-projectmeta-kit'``.
owner : `str`
    Repository owner component. Example: ``'lsst-sqre'``.
repo : `str`
    Repository name component. Example: ``'lsst-projectmeta-kit'``.
"""


# Detects a GitHub repo slug from a GitHub URL
GITHUB_SLUG_PATTERN = re.compile(
    r"https://github.com"
    r"/(?P<org>[a-zA-Z0-9\-_~%!$&'()*+,;=:@]+)"
    r"/(?P<name>[a-zA-Z0-9\-_~%!$&'()*+,;=:@]+)")


def parse_repo_slug_from_url(github_url):
    """Get the slug, <owner>/<repo_name>, for a GitHub repository from
    its URL.

    Parameters
    ----------
    github_url : `str`
        URL of a GitHub repository.

    Returns
    -------
    repo_slug : `RepoSlug`
        Repository slug with fields ``full``, ``owner``, and ``repo``.
        See `RepoSlug` for details.

    Raises
    ------
    RuntimeError
        Raised if the URL cannot be parsed.
    """
    match = GITHUB_SLUG_PATTERN.match(github_url)
    if not match:
        message = 'Could not parse GitHub slug from {}'.format(github_url)
        raise RuntimeError(message)

    _full = '/'.join((match.group('org'),
                      match.group('name')))
    return RepoSlug(_full, match.group('org'), match.group('name'))


def make_raw_content_url(repo_slug, git_ref, file_path):
    """Make a raw content (raw.githubusercontent.com) URL to a file.

    Parameters
    ----------
    repo_slug : `str` or `RepoSlug`
        The repository slug, formatted as either a `str` (``'owner/name'``)
        or a `RepoSlug` object (created by `parse_repo_slug_from_url`).
    git_ref : `str`
        The git ref: a branch name, commit hash, or tag name.
    file_path : `str`
        The POSIX path of the file in the repository tree.
    """
    if isinstance(repo_slug, RepoSlug):
        slug_str = repo_slug.full
    else:
        slug_str = repo_slug

    if file_path.startswith('/'):
        file_path = file_path.lstrip('/')

    template = 'https://raw.githubusercontent.com/{slug}/{git_ref}/{path}'
    return template.format(
        slug=slug_str,
        git_ref=git_ref,
        path=file_path)


def normalize_repo_root_url(url):
    """Normalize a GitHub URL into the root repository URL.

    Parameters
    ----------
    url : `str`
        A GitHub URL

    Returns
    -------
    url : `str`
        Normalized URL of a GitHub repository.

    Examples
    --------
    >>> normalize_repo_root_url('https://github.com/lsst/LDM-151.git')
    'https://github.com/lsst/LDM-151'
    """
    # Strip the .git extension, if present
    if url.endswith('.git'):
        url = url[:-4]
    return url



"""APIs for working with the GitHub v4 (GraphQL) API.

https://developer.github.com/v4/
"""

__all__ = ('github_request', 'GitHubQuery')

import os


async def github_request(session, api_token,
                         query=None, mutation=None, variables=None):
    """Send a request to the GitHub v4 (GraphQL) API.

    The request is asynchronous, with asyncio.

    Parameters
    ----------
    session : `aiohttp.ClientSession`
        Your application's aiohttp client session.
    api_token : `str`
        A GitHub personal API token. See the `GitHub personal access token
        guide`_.
    query : `str` or `GitHubQuery`
        GraphQL query string. If provided, then the ``mutation`` parameter
        should not be set. For examples, see the `GitHub guide to query and
        mutation operations`_.
    mutation : `str` or `GitHubQuery`
        GraphQL mutation string. If provided, then the ``query`` parameter
        should not be set. For examples, see the `GitHub guide to query and
        mutation operations`_.
    variables : `dict`
        GraphQL variables, as a JSON-compatible dictionary. This is only
        required if the ``query`` or ``mutation`` uses GraphQL variables.

    Returns
    -------
    data : `dict`
        Parsed JSON as a `dict` object.

    .. `GitHub personal access token guide`: https://ls.st/41d
    .. `GitHub guide to query and mutation operations`: https://ls.st/9s7
    """
    payload = {}
    if query is not None:
        payload['query'] = str(query)  # converts a GitHubQuery
    if mutation is not None:
        payload['mutation'] = str(mutation)  # converts a GitHubQuery
    if variables is not None:
        payload['variables'] = variables

    headers = {'Authorization': 'token {}'.format(api_token)}

    url = 'https://api.github.com/graphql'
    async with session.post(url, json=payload, headers=headers) as response:
        data = await response.json()

    return data


class GitHubQuery(object):
    """GitHub GraphQL (API v4) query string.

    Typically you'll use the `GitHubQuery.load` method to get a pre-made
    query string. Then use the `github_request` function to execute the query.

    Parameters
    ----------
    query : `str`
        GraphQL query or mutation string.
    name : `str`
        Description
    """

    def __init__(self, query, name=None):
        self.query = query
        self.name = name

    @classmethod
    def load(cls, query_name):
        """Load a pre-made query.

        These queries are distributed with lsstprojectmeta. See
        :file:`lsstrojectmeta/data/githubv4/README.rst` inside the
        package repository for details on available queries.

        Parameters
        ----------
        query_name : `str`
            Name of the query, such as ``'technote_repo'``.

        Returns
        -------
        github_query : `GitHubQuery
            A GitHub query or mutation object that you can pass to
            `github_request` to execute the request itself.
        """
        template_path = os.path.join(
            os.path.dirname(__file__),
            '../data/githubv4',
            query_name + '.graphql')

        with open(template_path) as f:
            query_data = f.read()

        return cls(query_data, name=query_name)

    def __str__(self):
        return self.query



"""Test LsstLatexDoc using sample data from LSE-63.tex.
"""

import datetime
import os
import pytest
import pytz
from lsstprojectmeta.tex.lsstdoc import LsstLatexDoc

TITLE = "LSST Data Quality Assurance Plan"

SHORT_TITLE = None

HTML_TITLE = "LSST Data Quality Assurance Plan\n"

HTML_SHORT_TITLE = None

PLAIN_TITLE = "LSST Data Quality Assurance Plan\n"

PLAIN_SHORT_TITLE = None

AUTHORS = [
    "Tony Tyson", "DQA Team", "Science Collaboration"
]

HTML_AUTHORS = [
    "Tony Tyson", "DQA Team", "Science Collaboration"
]

PLAIN_AUTHORS = [
    "Tony Tyson", "DQA Team", "Science Collaboration"
]

ABSTRACT = (
    r"LSST must supply trusted petascale data products. The mechanisms by "
    r"which the LSST project achieve this unprecedented level of data "
    r"quality will have spinoff to data-enabled science generally. This "
    r"document specifies high-level requirements for a LSST Data Quality "
    r"Assessment" + "\n"
    r"Framework, and defines the four levels of quality "
    r"assessment (QA) tools. Because this process involves system-wide "
    r"hardware and software, data QA must be defined at the System level. The "
    r"scope of this document is limited to the description of the overall "
    r"framework and the general requirements. It derives from the LSST "
    r"Science Requirements Document \citedsp{LPM-17}. A flow-down document "
    r"will describe detailed implementation of the QA, including the "
    r"algorithms.  In most cases the monitoring strategy, the development "
    r"path for these tools or the algorithms are known. Related documents "
    r"are: LSST System Requirements \citedsp{LSE-29}, Optimal Deployment "
    r"Parameters \citeds{Document-11624}, Observatory System Specifications "
    r"\citedsp{LSE-30}, Configuration Management Plan \citedsp{LPM-19}, "
    r"Project Quality Assurance Plan \citedsp{LPM-55}, Software Development "
    r"Plan \citedsp{LSE-16}, Camera Quality implementation Plan "
    r"\citedsp{LCA-227}, System Engineering Management Plan \citedsp{LSE-17}, "
    r"and the Operations Plan \citedsp{LPM-73}."
)

HTML_ABSTRACT = (
    "<p>LSST must supply trusted petascale data products. The mechanisms by "
    "which the LSST project achieve this unprecedented level of data "
    "quality will have spinoff to data-enabled science generally. This "
    "document specifies high-level requirements for a LSST Data Quality "
    "Assessment Framework, and defines the four levels of quality "
    "assessment (QA) tools. Because this process involves system-wide "
    "hardware and software, data QA must be defined at the System level. The "
    "scope of this document is limited to the description of the overall "
    "framework and the general requirements. It derives from the LSST "
    "Science Requirements Document [<a href=\"https://ls.st/LPM-17\">"
    "LPM-17</a>]. A flow-down document will describe detailed implementation "
    "of the QA, including the algorithms. In most cases the monitoring "
    "strategy, the development path for these tools or the algorithms are "
    "known. Related documents are: LSST System Requirements "
    "[<a href=\"https://ls.st/LSE-29\">LSE-29</a>], Optimal Deployment "
    "Parameters <a href=\"https://ls.st/Document-11624\">Document-11624</a>, "
    "Observatory System Specifications "
    "[<a href=\"https://ls.st/LSE-30\">LSE-30</a>], Configuration Management "
    "Plan [<a href=\"https://ls.st/LPM-19\">LPM-19</a>], "
    "Project Quality Assurance Plan "
    "[<a href=\"https://ls.st/LPM-55\">LPM-55</a>], "
    "Software Development Plan [<a href=\"https://ls.st/LSE-16\">LSE-16</a>], "
    "Camera Quality implementation Plan "
    "[<a href=\"https://ls.st/LCA-227\">LCA-227</a>], "
    "System Engineering Management Plan "
    "[<a href=\"https://ls.st/LSE-17\">LSE-17</a>], "
    "and the Operations Plan "
    "[<a href=\"https://ls.st/LPM-73\">LPM-73</a>].</p>\n"
)

PLAIN_ABSTRACT = (
    "LSST must supply trusted petascale data products. The mechanisms by "
    "which the LSST project achieve this unprecedented level of data "
    "quality will have spinoff to data-enabled science generally. This "
    "document specifies high-level requirements for a LSST Data Quality "
    "Assessment Framework, and defines the four levels of quality "
    "assessment (QA) tools. Because this process involves system-wide "
    "hardware and software, data QA must be defined at the System level. The "
    "scope of this document is limited to the description of the overall "
    "framework and the general requirements. It derives from the LSST "
    "Science Requirements Document [LPM-17]. A flow-down document will "
    "describe detailed implementation of the QA, including the algorithms. "
    "In most cases the monitoring strategy, the development path for these "
    "tools or the algorithms are known. Related documents are: "
    "LSST System Requirements [LSE-29], "
    "Optimal Deployment Parameters Document-11624, "
    "Observatory System Specifications [LSE-30], "
    "Configuration Management Plan [LPM-19], "
    "Project Quality Assurance Plan [LPM-55], "
    "Software Development Plan [LSE-16], "
    "Camera Quality implementation Plan [LCA-227], "
    "System Engineering Management Plan [LSE-17], "
    "and the Operations Plan [LPM-73].\n"
)

IS_DRAFT = False

HANDLE = 'LSE-63'

SERIES = 'LSE'

SERIAL = '63'

ATTRIBUTES = [
    ('title', TITLE),
    ('short_title', SHORT_TITLE),
    ('html_title', HTML_TITLE),
    ('html_short_title', HTML_SHORT_TITLE),
    ('plain_title', PLAIN_TITLE),
    ('plain_short_title', PLAIN_SHORT_TITLE),
    ('authors', AUTHORS),
    ('html_authors', HTML_AUTHORS),
    ('plain_authors', PLAIN_AUTHORS),
    ('abstract', ABSTRACT),
    ('html_abstract', HTML_ABSTRACT),
    ('plain_abstract', PLAIN_ABSTRACT),
    ('is_draft', IS_DRAFT),
    ('handle', HANDLE),
    ('series', SERIES),
    ('serial', SERIAL),
]

JSONLD = {
    '@context': [
        "https://raw.githubusercontent.com/codemeta/codemeta/2.0-rc/"
        "codemeta.jsonld",
        "http://schema.org"],
    '@type': ['Report', 'SoftwareSourceCode'],
    'language': 'TeX',
    'reportNumber': HANDLE,
    'name': PLAIN_TITLE,
    'description': PLAIN_ABSTRACT,
    'author': [{'@type': 'Person', 'name': author_name}
               for author_name in PLAIN_AUTHORS],
}


@pytest.fixture
def lsstdoc():
    tex_path = os.path.join(os.path.dirname(__file__),
                            'data',
                            'LSE-63.tex')
    return LsstLatexDoc.read(tex_path)


@pytest.mark.parametrize('attribute,expected', ATTRIBUTES)
def test_attribute(lsstdoc, attribute, expected):
    assert getattr(lsstdoc, attribute) == expected


def test_revision_date(lsstdoc):
    r"""LSE-63 uses a set value for \date."""
    expected_datetime = datetime.datetime(2017, 5, 17, 7, 0, tzinfo=pytz.utc)
    assert lsstdoc.revision_datetime == expected_datetime
    assert lsstdoc.revision_datetime_source == 'tex'


def test_jsonld(lsstdoc):
    jsonld = lsstdoc.build_jsonld()
    for key, value in JSONLD.items():
        assert jsonld[key] == value
    assert jsonld['dateModified'] == lsstdoc.revision_datetime



"""Test LsstLatexDoc using sample data from LDM-151.tex.
"""

import datetime
import os
import pytest
import pytz
from lsstprojectmeta.tex.lsstdoc import LsstLatexDoc

TITLE = "Data Management Science Pipelines Design"

SHORT_TITLE = None

HTML_TITLE = "Data Management Science Pipelines Design\n"

HTML_SHORT_TITLE = None

PLAIN_TITLE = "Data Management Science Pipelines Design\n"

PLAIN_SHORT_TITLE = None

AUTHORS = [
    "J.D. Swinbank",
    "T. Axelrod", "A.C. Becker", "J. Becla", "E. Bellm",
    "J.F. Bosch", "H. Chiang", "D.R. Ciardi", "A.J. Connolly",
    "G.P. Dubois-Felsmann", "F. Economou", "M. Fisher-Levine", "M. Graham",
    r"\v{Z}. Ivezi\'c", r"M. Juri\'c",
    "T. Jenness", "R.L. Jones", "J. Kantor", "S. Krughoff", "K-T. Lim",
    "R.H. Lupton", "F. Mueller", "D. Petravick", "P.A. Price",
    "D.J. Reiss", "D. Shaw", "C. Slater", "M. Wood-Vasey", "X. Wu",
    "P. Yoachim", r"\emph{for the LSST Data Management}"
]

HTML_AUTHORS = [
    "J.D. Swinbank",
    "T. Axelrod", "A.C. Becker", "J. Becla", "E. Bellm",
    "J.F. Bosch", "H. Chiang", "D.R. Ciardi", "A.J. Connolly",
    "G.P. Dubois-Felsmann", "F. Economou", "M. Fisher-Levine",
    "M. Graham", "Ž. Ivezić", "M. Jurić",
    "T. Jenness", "R.L. Jones", "J. Kantor", "S. Krughoff",
    "K-T. Lim", "R.H. Lupton", "F. Mueller", "D. Petravick",
    "P.A. Price", "D.J. Reiss", "D. Shaw", "C. Slater",
    "M. Wood-Vasey", "X. Wu", "P. Yoachim",
    "<em>for the LSST Data Management</em>"
]

PLAIN_AUTHORS = [
    "J.D. Swinbank",
    "T. Axelrod", "A.C. Becker", "J. Becla", "E. Bellm",
    "J.F. Bosch", "H. Chiang", "D.R. Ciardi", "A.J. Connolly",
    "G.P. Dubois-Felsmann", "F. Economou", "M. Fisher-Levine",
    "M. Graham", "Ž. Ivezić", "M. Jurić",
    "T. Jenness", "R.L. Jones", "J. Kantor", "S. Krughoff",
    "K-T. Lim", "R.H. Lupton", "F. Mueller", "D. Petravick",
    "P.A. Price", "D.J. Reiss", "D. Shaw", "C. Slater",
    "M. Wood-Vasey", "X. Wu", "P. Yoachim",
    "_for the LSST Data Management_"
]

ABSTRACT = (
    r"The LSST Science Requirements Document (the LSST \SRD) specifies a set "
    r"of data product guidelines, designed to support science goals "
    r"envisioned to be enabled by the LSST observing program." + "\n"
    r"Following these guidlines, the details of these data products have "
    r"been described in the LSST Data Products Definition Document (\DPDD), "
    r"and captured in a formal flow-down from the \SRD via the LSST System "
    r"Requirements (\LSR), Observatory System Specifications (\OSS), to the "
    r"Data Management System Requirements (\DMSR)." + "\n"
    r"The LSST Data Management subsystem's responsibilities include the "
    r"design, implementation, deployment and execution of software pipelines "
    r"necessary to generate these data products. This document describes the "
    r"design of the scientific aspects of those pipelines."
)

HTML_ABSTRACT = (
    '<p>The LSST Science Requirements Document (the LSST '
    '<span><a href="https://docushare.lsst.org/docushare/dsweb/Get/LPM-17">'
    'SRD</a></span>) specifies a set of data product guidelines, designed to '
    'support science goals envisioned to be enabled by the LSST observing '
    'program. Following these guidlines, the details of these data products '
    'have been described in the LSST Data Products Definition Document '
    '(<span><a href="https://docushare.lsst.org/docushare/dsweb/Get/'
    'LSE-163">DPDD</a></span>), and captured in a formal flow-down from '
    'the <span><a href="https://docushare.lsst.org/docushare/dsweb/Get/'
    'LPM-17">SRD</a></span>via the LSST System Requirements (<span>'
    '<a href="https://docushare.lsst.org/docushare/dsweb/Get/LSE-29">'
    'LSR</a></span>), Observatory System Specifications (<span>'
    '<a href="https://docushare.lsst.org/docushare/dsweb/Get/LSE-30">'
    'OSS</a></span>), to the Data Management System Requirements '
    '(<span><a href="https://docushare.lsst.org/docushare/dsweb/Get/'
    'LSE-61">DMSR</a></span>). The LSST Data Management subsystem’s '
    'responsibilities include the design, implementation, deployment and '
    'execution of software pipelines necessary to generate these data '
    'products. This document describes the design of the scientific aspects '
    'of those pipelines.</p>\n'
)

# NOTE there's an issue with getting spaces to appear after commands.
# xspace doesn't work for us.
PLAIN_ABSTRACT = (
    "The LSST Science Requirements Document (the LSST SRD) specifies a set "
    "of data product guidelines, designed to support science goals "
    "envisioned to be enabled by the LSST observing program. "
    "Following these guidlines, the details of these data products have "
    "been described in the LSST Data Products Definition Document (DPDD), "
    "and captured in a formal flow-down from the SRDvia the LSST System "
    "Requirements (LSR), Observatory System Specifications (OSS), to the "
    "Data Management System Requirements (DMSR). "
    "The LSST Data Management subsystem’s responsibilities include the "
    "design, implementation, deployment and execution of software pipelines "
    "necessary to generate these data products. This document describes the "
    "design of the scientific aspects of those pipelines.\n"
)

IS_DRAFT = False

HANDLE = 'LDM-151'

SERIES = 'LDM'

SERIAL = '151'

ATTRIBUTES = [
    ('title', TITLE),
    ('short_title', SHORT_TITLE),
    ('html_title', HTML_TITLE),
    ('html_short_title', HTML_SHORT_TITLE),
    ('plain_title', PLAIN_TITLE),
    ('plain_short_title', PLAIN_SHORT_TITLE),
    ('authors', AUTHORS),
    ('html_authors', HTML_AUTHORS),
    ('plain_authors', PLAIN_AUTHORS),
    ('abstract', ABSTRACT),
    ('html_abstract', HTML_ABSTRACT),
    ('plain_abstract', PLAIN_ABSTRACT),
    ('is_draft', IS_DRAFT),
    ('handle', HANDLE),
    ('series', SERIES),
    ('serial', SERIAL),
]

JSONLD = {
    '@context': [
        "https://raw.githubusercontent.com/codemeta/codemeta/2.0-rc/"
        "codemeta.jsonld",
        "http://schema.org"],
    '@type': ['Report', 'SoftwareSourceCode'],
    'language': 'TeX',
    'reportNumber': HANDLE,
    'name': PLAIN_TITLE,
    'description': PLAIN_ABSTRACT,
    'author': [{'@type': 'Person', 'name': author_name}
               for author_name in PLAIN_AUTHORS],
}


@pytest.fixture
def lsstdoc():
    tex_path = os.path.join(os.path.dirname(__file__),
                            'data',
                            'LDM-151',
                            'LDM-151.tex')
    return LsstLatexDoc.read(tex_path)


@pytest.mark.parametrize('attribute,expected', ATTRIBUTES)
def test_attribute(lsstdoc, attribute, expected):
    assert getattr(lsstdoc, attribute) == expected


def test_revision_date(lsstdoc):
    r"""LDM-151 uses a set value for \date."""
    expected_datetime = datetime.datetime(2017, 7, 19, 7, 0, tzinfo=pytz.utc)
    assert lsstdoc.revision_datetime == expected_datetime
    assert lsstdoc.revision_datetime_source == 'tex'


def test_jsonld(lsstdoc):
    jsonld = lsstdoc.build_jsonld()
    for key, value in JSONLD.items():
        assert jsonld[key] == value
    assert jsonld['dateModified'] == lsstdoc.revision_datetime



"""Tests for lsstprojectmeta.github.urls.
"""

import pytest
from lsstprojectmeta.github.urls import (
    parse_repo_slug_from_url, RepoSlug, make_raw_content_url,
    normalize_repo_root_url)


@pytest.mark.parametrize(
    'url,expected',
    [('https://github.com/lsst-sqre/lsst-projectmeta-kit',
     ('lsst-sqre/lsst-projectmeta-kit', 'lsst-sqre', 'lsst-projectmeta-kit')),
     ('https://github.com/lsst/LDM-151',
     ('lsst/LDM-151', 'lsst', 'LDM-151')),
     ('https://github.com/lsst-dm/dmtn-030',
     ('lsst-dm/dmtn-030', 'lsst-dm', 'dmtn-030')),
     ])
def test_parse_repo_slug_from_url(url, expected):
    repo_slug = parse_repo_slug_from_url(url)
    assert repo_slug.full == expected[0]
    assert repo_slug.owner == expected[1]
    assert repo_slug.repo == expected[2]


@pytest.mark.parametrize(
    'inputs,expected',
    [(('lsst-sqre/sqr-020', 'master', 'metadata.yaml'),
      'https://raw.githubusercontent.com/lsst-sqre/sqr-020/'
      'master/metadata.yaml'),
     ((RepoSlug('lsst-sqre/sqr-020', 'lsst-sqre', 'sqr-020'),
       'master', 'metadata.yaml'),
      'https://raw.githubusercontent.com/lsst-sqre/sqr-020/'
      'master/metadata.yaml')]
    )
def test_make_raw_content_url(inputs, expected):
    assert make_raw_content_url(*inputs) == expected


@pytest.mark.parametrize(
    'url,expected', [
        ('https://github.com/lsst/LDM-151.git',
         'https://github.com/lsst/LDM-151')
    ])
def test_normalize_repo_root_url(url, expected):
    assert normalize_repo_root_url(url) == expected



"""GitHub integration installation authentication.

https://developer.github.com/early-access/integrations/authentication/
"""

import datetime

import jwt
import requests

__all__ = ['create_jwt', 'get_installation_token']


def get_installation_token(installation_id, integration_jwt):
    """Create a GitHub token for an integration installation.

    Parameters
    ----------
    installation_id : `int`
        Installation ID. This is available in the URL of the integration's
        **installation** ID.
    integration_jwt : `bytes`
        The integration's JSON Web Token (JWT). You can create this with
        `create_jwt`.

    Returns
    -------
    token_obj : `dict`
        GitHub token object. Includes the fields:

        - ``token``: the token string itself.
        - ``expires_at``: date time string when the token expires.

    Example
    -------
    The typical workflow for authenticating to an integration installation is:

    .. code-block:: python

       from dochubadapter.github import auth
       jwt = auth.create_jwt(integration_id, private_key_path)
       token_obj = auth.get_installation_token(installation_id, jwt)
       print(token_obj['token'])

    Notes
    -----
    See
    https://developer.github.com/early-access/integrations/authentication/#as-an-installation
    for more information
    """
    api_root = 'https://api.github.com'
    url = '{root}/installations/{id_:d}/access_tokens'.format(
        api_root=api_root,
        id_=installation_id)

    headers = {
        'Authorization': 'Bearer {0}'.format(integration_jwt.decode('utf-8')),
        'Accept': 'application/vnd.github.machine-man-preview+json'
    }

    resp = requests.post(url, headers=headers)
    resp.raise_for_status()
    return resp.json()


def create_jwt(integration_id, private_key_path):
    """Create a JSON Web Token to authenticate a GitHub Integration or
    installation.

    Parameters
    ----------
    integration_id : `int`
        Integration ID. This is available from the GitHub integration's
        homepage.
    private_key_path : `str`
        Path to the integration's private key (a ``.pem`` file).

    Returns
    -------
    jwt : `bytes`
        JSON Web Token that is good for 9 minutes.

    Notes
    -----
    The JWT is encoded with the RS256 algorithm. It includes a payload with
    fields:

    - ``'iat'``: The current time, as an `int` timestamp.
    - ``'exp'``: Expiration time, as an `int timestamp. The expiration
      time is set of 9 minutes in the future (maximum allowance is 10 minutes).
    - ``'iss'``: The integration ID (`int`).

    For more information, see
    https://developer.github.com/early-access/integrations/authentication/.
    """
    integration_id = int(integration_id)

    with open(private_key_path, 'rb') as f:
        cert_bytes = f.read()

    now = datetime.datetime.now()
    expiration_time = now + datetime.timedelta(minutes=9)
    payload = {
        # Issued at time
        'iat': int(now.timestamp()),
        # JWT expiration time (10 minute maximum)
        'exp': int(expiration_time.timestamp()),
        # Integration's GitHub identifier
        'iss': integration_id
    }

    return jwt.encode(payload, cert_bytes, algorithm='RS256')



"""Test LsstLatexDoc using sample data from DMTN-036.tex.
"""

import datetime
import os
import pytest
from lsstprojectmeta.tex.lsstdoc import LsstLatexDoc

TITLE = (
    "jointcal: Simultaneous Astrometry \\& Photometry for thousands "
    "of\nExposures with Large CCD Mosaics"
)

SHORT_TITLE = "jointcal"

HTML_TITLE = (
    "jointcal: Simultaneous Astrometry &amp; Photometry for thousands of "
    "Exposures with Large CCD Mosaics\n"
)

HTML_SHORT_TITLE = "jointcal\n"

PLAIN_TITLE = (
    "jointcal: Simultaneous Astrometry & Photometry for thousands of "
    "Exposures with Large CCD Mosaics\n"
)

PLAIN_SHORT_TITLE = "jointcal\n"

AUTHORS = [
    "John Parejko (University of Washington)",
    "Pierre Astier (LPNHE/IN2P3/CNRS Paris)"
]

HTML_AUTHORS = [
    "John Parejko (University of Washington)",
    "Pierre Astier (LPNHE/IN2P3/CNRS Paris)"
]

PLAIN_AUTHORS = [
    "John Parejko (University of Washington)",
    "Pierre Astier (LPNHE/IN2P3/CNRS Paris)"
]

HTML_ABSTRACT = (
    "<p>The jointcal package simultaneously optimizes the astrometric and "
    "photometric calibrations of a set of astronomical images. In "
    "principle and often in practice, this approach produces distortion "
    "and thoroughput models which are more precise than when fitted "
    "independently. This is especially true when the images are deeper "
    "than the astrometric reference catalogs. In the “Astromatic” "
    "software suite, this simultaneous astrometry functionality is "
    "fulfilled by “SCAMP”. The code we describe here has similar aims, "
    "but follows a slightly different route. Jointcal is built on top of "
    "the the LSST Data Management software stack.</p>\n"
)

PLAIN_ABSTRACT = (
    "The jointcal package simultaneously optimizes the astrometric and "
    "photometric calibrations of a set of astronomical images. In "
    "principle and often in practice, this approach produces distortion "
    "and thoroughput models which are more precise than when fitted "
    "independently. This is especially true when the images are deeper "
    "than the astrometric reference catalogs. In the “Astromatic” "
    "software suite, this simultaneous astrometry functionality is "
    "fulfilled by “SCAMP”. The code we describe here has similar aims, "
    "but follows a slightly different route. Jointcal is built on top of "
    "the the LSST Data Management software stack.\n"
)

IS_DRAFT = True

HANDLE = 'DMTN-036'

SERIES = 'DMTN'

SERIAL = '036'

ATTRIBUTES = [
    ('title', TITLE),
    ('short_title', SHORT_TITLE),
    ('html_title', HTML_TITLE),
    ('html_short_title', HTML_SHORT_TITLE),
    ('plain_title', PLAIN_TITLE),
    ('plain_short_title', PLAIN_SHORT_TITLE),
    ('authors', AUTHORS),
    ('html_authors', HTML_AUTHORS),
    ('plain_authors', PLAIN_AUTHORS),
    ('html_abstract', HTML_ABSTRACT),
    ('plain_abstract', PLAIN_ABSTRACT),
    ('is_draft', IS_DRAFT),
    ('handle', HANDLE),
    ('series', SERIES),
    ('serial', SERIAL)
]

JSONLD = {
    '@context': [
        "https://raw.githubusercontent.com/codemeta/codemeta/2.0-rc/"
        "codemeta.jsonld",
        "http://schema.org"],
    '@type': ['Report', 'SoftwareSourceCode'],
    'language': 'TeX',
    'reportNumber': HANDLE,
    'name': PLAIN_TITLE,
    'description': PLAIN_ABSTRACT,
    'author': [{'@type': 'Person', 'name': author_name}
               for author_name in PLAIN_AUTHORS],
}


@pytest.fixture
def lsstdoc():
    tex_path = os.path.join(os.path.dirname(__file__), 'data', 'DMTN-036.tex')
    return LsstLatexDoc.read(tex_path)


@pytest.mark.parametrize('attribute,expected', ATTRIBUTES)
def test_attribute(lsstdoc, attribute, expected):
    assert getattr(lsstdoc, attribute) == expected


def test_revision_date(lsstdoc):
    r"""DMTN-036 is a draft, so it falls back to Git."""
    assert isinstance(lsstdoc.revision_datetime, datetime.datetime)
    assert lsstdoc.revision_datetime_source == 'git'


def test_jsonld(lsstdoc):
    jsonld = lsstdoc.build_jsonld()
    for key, value in JSONLD.items():
        assert jsonld[key] == value
    assert jsonld['dateModified'] == lsstdoc.revision_datetime
